#!/usr/bin/bash

# Copyright © 2024, Oracle and/or its affiliates.  All rights reserved.
# Licensed under the Universal Permissive License v 1.0 as shown at https://oss.oracle.com/licenses/upl/

set -x

oci-fss-setup-prefix.sh

CACHE_DIR=/tmp/oci-fss-utils.d
gc_junk_due_to_reboot() {
    # remove junk oci-fss-[0-9]+.service due to reboot
    junk=$(systemctl list-units --all oci-fss-*.service | egrep 'oci-fss-[0-9]+\.service' -o)

    for unit in $junk; do
        systemctl stop $unit || true
        systemctl disable $unit || true
    done

    # This find is here to support update/upgrade...older versions once stored systemd service files here.
    find /usr/lib/systemd/system -regextype posix-egrep -regex '/usr/lib/systemd/system/oci-fss-[0-9]+\.service' -delete
    find /etc/systemd/system -regextype posix-egrep -regex '/etc/systemd/system/oci-fss-[0-9]+\.service' -delete

    systemctl daemon-reload
    systemctl reset-failed

    rm -rf $CACHE_DIR
}

# on reboot (we don't have ns1), we gc the junk oci-fss-[0-9]+
if ! /sbin/ip netns exec ns1 true 2>/dev/null ; then
    gc_junk_due_to_reboot
fi

# This tmp directory is used by mount.oci-fss and oci-fss-gc to quickly
# find forwarders associated to a mount target ip, or systemd service file.
(umask 0022 && mkdir -p $CACHE_DIR)

# Create namespace
/sbin/ip netns add ns1

# Create veth link.
/sbin/ip link add x-eth0 type veth peer name x-peer0

/sbin/ip link set x-eth0 mtu 65535
/sbin/ip link set x-peer0 mtu 65535

# Add peer to NS.
/sbin/ip link set x-peer0 netns ns1

/sbin/ip link set x-eth0 up
/sbin/ip netns exec ns1 /sbin/ip link set x-peer0 up

PREFIX_FILE=/etc/oci-fss-utils.d/prefix.txt
PREFIX=$(cat $PREFIX_FILE)
PROTOCOL=$(oci-fss-validate-cidr $PREFIX)

if [[ $PROTOCOL == ipv6 ]] ; then

    PREFIX=${PREFIX/::\/64}

    # Add this address to x-eth0 and magically everything with a destination to
    # this CIRD will route to x-eth0.
    /sbin/ip addr add ${PREFIX}::/64 dev x-eth0

    # Add these addrs from the CIDR to x-peer0
    for i in $(seq 1 254) ; do
        /sbin/ip netns exec ns1 /sbin/ip addr add $(printf "%s::%x/64" $PREFIX $i) dev x-peer0
    done

elif [[ $PROTOCOL == ipv4 ]] ; then

    PREFIX=${PREFIX/.0\/24}

    # Add this address to x-eth0 and magically everything with a destination to
    # this CIRD will route to x-eth0.
    /sbin/ip addr add ${PREFIX}.0/24 dev x-eth0

    # Add these addrs from the CIDR to x-peer0
    for i in $(seq 1 254) ; do
        /sbin/ip netns exec ns1 /sbin/ip addr add $(printf "%s.%d/32" $PREFIX $i) dev x-peer0
    done
    /sbin/ip netns exec ns1 /sbin/ip route add ${PREFIX}.0/32 dev x-peer0

else

    echo ERROR!!!  Invalid prefix in $PREFIX_FILE.  Valid choices are \"ipv6\" and \"ipv4\". 1>&2
    exit 1

fi

# Setup IP address of x-peer
/sbin/ip netns exec ns1 /sbin/ip link set x-peer0 up

instance_metadata=$(curl -m 10 -H "Authorization: Bearer Oracle" -s http://169.254.169.254/opc/v2/instance/)
if [ "$instance_metadata" == "" ]; then
   echo "Failed to make oci instance metadata call. Assuming running from on-prem instance."
fi
 
regionInfo=$(echo $instance_metadata | jq '.regionInfo')
if [ "$regionInfo" != "" ]; then

   realmDomainComponent=$(echo $regionInfo | jq '.realmDomainComponent' | tr -d '"')
   if [ "$realmDomainComponent" == "" ]; then
      echo "Failed to find the realmDomainComponent from oci region information"
      exit 1
   fi

   update_wildcards=$(jq ".wildcards = (.wildcards += [\".$realmDomainComponent\"] | .wildcards | unique)" /etc/oci-fss-utils.d/config.json)
   echo "$update_wildcards" | jq '.' > /etc/oci-fss-utils.d/config.json
fi

mkdir -p $(dirname $(jq -sr ".[0].caFile" /etc/oci-fss-utils.d/config.json))
